/**
  ******************************************************************************
  * @file    main.c
  * @author  MCD Application Team
  * @version V1.0.0
  * @date    31-October-2011
  * @brief   Main program body
  ******************************************************************************
  * @attention
  *
  * THE PRESENT FIRMWARE WHICH IS FOR GUIDANCE ONLY AIMS AT PROVIDING CUSTOMERS
  * WITH CODING INFORMATION REGARDING THEIR PRODUCTS IN ORDER FOR THEM TO SAVE
  * TIME. AS A RESULT, STMICROELECTRONICS SHALL NOT BE HELD LIABLE FOR ANY
  * DIRECT, INDIRECT OR CONSEQUENTIAL DAMAGES WITH RESPECT TO ANY CLAIMS ARISING
  * FROM THE CONTENT OF SUCH FIRMWARE AND/OR THE USE MADE BY CUSTOMERS OF THE
  * CODING INFORMATION CONTAINED HEREIN IN CONNECTION WITH THEIR PRODUCTS.
  *
  * <h2><center>&copy; COPYRIGHT 2011 STMicroelectronics</center></h2>
  ******************************************************************************
  */

/* Includes ------------------------------------------------------------------*/

#include <stdio.h>
#include "main.h"


/* Exported functions ---------------------------------------------------------*/
extern  uint8_t CAN_Send(CAN_TypeDef *CAN,CanTxMsg *Message);

/* Global Variables ---------------------------------------------------------*/
xQueueHandle xQueueCAN2,xQueueCAN1,xQueueSendArray;

/* Private typedef -----------------------------------------------------------*/
/* Private define ------------------------------------------------------------*/

/*--------------- Tasks Priority -------------*/
#define TASK_PRIO    ( tskIDLE_PRIORITY + 1 )


/* Private macro -------------------------------------------------------------*/
/* Private variables ---------------------------------------------------------*/

 uint8_t array_receive[2000];
/* Private function prototypes -----------------------------------------------*/
static void LED_Init(void);

/* Private functions ---------------------------------------------------------*/

/**
  * @brief  CAN1 receive Array and toggles LED orange it consists of two states 0=reset and 1=write data
  * @param  pvParameters not used
  * @retval None
  */
void vTask_ArrayReceive(void * pvParameters)
{
	static uint16_t i,j;
	static uint8_t rec_state=0;
    CanRxMsg RxMessage;
    static uint8_t buttonstate=0;
    uint8_t transmitstatus=0;


      for( ;; )
      {

    	 /*wait till a Message was received*/
    	xQueueReceive( xQueueCAN1, &( RxMessage ), portMAX_DELAY );

    	/*was it a start, data or end Id otherwise do not care*/
    	switch(RxMessage.StdId)
    	{
    	/*a start Id*/
    	case 0x100:

    		/*we are in reset state*/
    		if(rec_state==0)
    			{
    				rec_state=1;	//ready to write something
    				i=0;
    			}
    		else
    			{
    				rec_state=0;	//error we were already in write data state
    				i=0;
    			}

    		break;

    	/*a data Id*/
    	case 0x101:

    		/*we are in data write state*/
    		if(rec_state==1)
    		{
    			/*write data*/
    			for(j=0;j<RxMessage.DLC;j++)
    				array_receive[i+j]=RxMessage.Data[j];

    			/*ad the number of data to the index*/
    			i+=RxMessage.DLC;
    		}
    		else
    			/*error reset state*/
    			rec_state=0;

    		break;

    	/*a end Id*/
    	case 0x102:
    		/*we are in data write state */
    		if(rec_state==1)
    			{
    				rec_state=0;
    				i=0;
    				GPIO_ToggleBits(GPIOD, GPIO_Pin_12);
    			}

    		break;

    	/*everything else do not care*/
    	default:
    		break;

    		}


      }

}

/**
  * @brief  Resets the red LED
  * @param  pvParameters not used
  * @retval None
  */
void vTask_LEDReset(void * pvParameters)
{

      for( ;; )
      {

    	vTaskDelay(50);

    	GPIO_ResetBits(GPIOD, GPIO_Pin_14);

        vTaskDelay(50);

      }

}


/**
  * @brief  Main program.
  * @param  None
  * @retval None
  */
int main(void)
{
	int8_t err;
	CanRxMsg RxMessage;

	SystemInit();

  /*!< At this stage the microcontroller clock setting is already configured to
       168 MHz, this is done through SystemInit() function which is called from
       startup file (startup_stm32f4xx.s) before to branch to application main.
       To reconfigure the default setting of SystemInit() function, refer to
       system_stm32f4xx.c file
     */

  /*Initialize Leds */
  LED_Init();

  /*Configure the Interrupt*/
  InterruptCAN_Init();

  /*Initialize Can module */
  CAN_Initialisierung();

  xQueueCAN1 = xQueueCreate( 1, sizeof( RxMessage ) );

  if( xQueueCAN1 == 0 )
  	  {
	  	  while(1)
	  		  ;
  	  }


  /* Create Tasks */
  err=xTaskCreate( vTask_ArrayReceive, NULL, 1000, NULL, TASK_PRIO, NULL);
  if(err!=1)
	  while(1);
  err=xTaskCreate( vTask_LEDReset, NULL, 130, NULL, TASK_PRIO, NULL);
  if(err!=1)
	  while(1);


  /*Enable CAN Interrupt*/
  CAN_ITConfig(CAN2, CAN_IT_FMP0, ENABLE);
 /*Enable CAN Interrupt*/
  CAN_ITConfig(CAN1, CAN_IT_FMP1, ENABLE);

  /* Start scheduler */
  vTaskStartScheduler();

  /* We should never get here as control is now taken by the scheduler */
  for( ;; );
}



/**
  * @brief  Initializes the LEDs resources.
  * @param  None
  * @retval None
  */
void LED_Init(void)
{
	GPIO_InitTypeDef GPIO_InitStructure;

  /* Initialize Discovery's LEDs */
  RCC_AHB1PeriphClockCmd(RCC_AHB1Periph_GPIOD, ENABLE);

  /* Configure PD15 in output pushpull mode */
  GPIO_InitStructure.GPIO_Pin = GPIO_Pin_12|GPIO_Pin_13|  GPIO_Pin_14;
  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_OUT;
  GPIO_InitStructure.GPIO_OType = GPIO_OType_PP;
  GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
  GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_NOPULL;
  GPIO_Init(GPIOD, &GPIO_InitStructure);

  RCC_AHB1PeriphClockCmd(RCC_AHB1Periph_GPIOA, ENABLE);

  /* Configure PA0 input pulldown mode*/
  GPIO_InitStructure.GPIO_Pin = GPIO_Pin_0;
  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_IN;
  GPIO_InitStructure.GPIO_OType = GPIO_OType_PP;
  GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
  GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_DOWN;

  GPIO_Init(GPIOA, &GPIO_InitStructure);

}






/******************* (C) COPYRIGHT 2011 STMicroelectronics *****END OF FILE****/
